"use strict";
var ts = require("typescript");
var exceptions_1 = require("tsoa/dist/metadataGeneration/exceptions");
var statusCodes_1 = require("tsoa/dist/utils/statusCodes");
let routingControllerDecoratorPlugin = {
    exampleIdentifiers: [],
    methodActionIdentifiers: ['Get', 'Post', 'Patch', 'Put', 'Delete', 'Head', 'Method'],
    parameterIdentifiers: [
        { name: 'Req', type: 'request' },
        { name: 'Body', type: 'body' },
        { name: 'BodyParam', type: 'body-prop' },
        { name: 'HeaderParam', type: 'header' },
        { name: 'QueryParam', type: 'query' },
        { name: 'Param', type: 'path' },
    ],
    produceIdentifiers: ['ContentType'],
    responseIdentifiers: ['HttpCode', 'OnNull', 'OnUndefined', 'Redirect'],
    routeIdentifiers: ['Controller', 'JsonController'],
    securityIdentifiers: ['Authorize'],
    tagIdentifiers: [],
    getExample(expression) {
        throw new exceptions_1.GenerateMetadataError(expression, 'Example is not supported.');
    },
    getRoutePrefix(expression) {
        const value = expression.arguments[0];
        return value ? value.text.replace(/^\//, '') : '';
    },
    getMethodAction(expression) {
        const decorator = expression.expression;
        let decoratorArgument = expression.arguments[0];
        let method = decorator.text.toLowerCase();
        if (decorator.text === 'Method') {
            method = decoratorArgument.text.toLowerCase();
            decoratorArgument = expression.arguments[1];
        }
        if (decoratorArgument && decoratorArgument.kind !== ts.SyntaxKind.StringLiteral) {
            throw new exceptions_1.GenerateMetadataError(expression, decorator.text + ' argument must be string.');
        }
        return {
            method,
            path: decoratorArgument ? `${decoratorArgument.text.replace(/:(\w+)\b/g, '{$1}')}` : ''
        };
    },
    getMethodResponse(expression, returnType) {
        const decorator = expression.expression;
        let code = 200;
        if (decorator.text === 'Redirect') {
            code = 302;
        }
        else {
            const decoratorArgument = expression.arguments[0];
            if (decoratorArgument && decoratorArgument.kind !== ts.SyntaxKind.NumericLiteral) {
                throw new exceptions_1.GenerateMetadataError(expression, decorator.text + ' argument must be number.');
            }
            code = +decoratorArgument.text;
        }
        return {
            code,
            description: statusCodes_1.getPhase(code),
            name: code.toString(),
            schema: code < 300 ? returnType : undefined
        };
    },
    getMethodSecurities(expression) {
        let name = '';
        const decoratorArgument = expression.arguments[0];
        if (decoratorArgument.kind === ts.SyntaxKind.StringLiteral) {
            name = decoratorArgument.text;
        }
        else if (decoratorArgument.kind === ts.SyntaxKind.ArrayLiteralExpression) {
            const arrayLiteralExpression = decoratorArgument;
            if (arrayLiteralExpression.elements.every(element => element.kind === ts.SyntaxKind.StringLiteral)) {
                name = arrayLiteralExpression.elements.map((element) => element.text).join(', ');
            }
        }
        if (!name) {
            throw new exceptions_1.GenerateMetadataError(expression, 'Authorize argument must be string or string array.');
        }
        return { name };
    },
    getMethodTags(expression) {
        throw new exceptions_1.GenerateMetadataError(expression, 'Tags is not supported.');
    },
    getProduce(expression) {
        const value = expression.arguments[0];
        return value.text;
    }
};
module.exports = routingControllerDecoratorPlugin;
